/***************************************************************************

    JPM IMPACT

    based on driver by Phil Bennett

    Known issues:
        * I/O documentation for lamps, reels, meters etc is possibly incorrect.
        * DUART emulation is very simplistic, in progress.
        * Digital volume control is not emulated.

****************************************************************************

    Memory map (preliminary)

****************************************************************************

    ========================================================================
    Main CPU (68000)
    ========================================================================
    000000-0FFFFF   R     xxxxxxxx xxxxxxxx   Program ROM bank 1
    100000-1FFFFF   R     xxxxxxxx xxxxxxxx   Program ROM bank 2
    400000-403FFF   R/W   xxxxxxxx xxxxxxxx   Program RAM (battery-backed)
    480000-48001F   R/W   -------- xxxxxxxx   MC68681 DUART 1
    480020-480033   R     -------- xxxxxxxx   Inputs
    480041          R     -xxxxxxx xxxxxxxx   Reel optos
    480060-480067   R/W   -------- xxxxxxxx   D71055C? 8255 PPI?
    480080-480081     W   -------- xxxxxxxx   uPD7559 communications
    480082-480083     W   -------- xxxxxxxx   Sound control
                          -------- -------x      (uPD7759 reset)
                          -------- -----xx-      (ROM A18-A17)
                          -------- ---x----      (X9C103 /INC)
                          -------- --x-----      (X9C103 U/#D)
                          -------- -x------      (X9C103 /CS)
    480084-480085   R     -------- xxxxxxxx   uPD7759 communications
    4800A0-4800AF     W   xxxxxxxx xxxxxxxx   Lamps?
    4800E0-4800E1     W   xxxxxxxx xxxxxxxx   Reset and status LEDs?
    4801DC-4801DD   R     -------- xxxxxxxx   Unknown
    4801DE-4801DF   R     -------- xxxxxxxx   Unknown
    4801E0-4801FF   R/W   -------- xxxxxxxx   MC68681 DUART 2 (on ROM PCB)
    800000-800007   R/W   xxxxxxxx xxxxxxxx   TMS34010 interface
    C00000-CFFFFF   R     xxxxxxxx xxxxxxxx   Question ROM bank 1
    D00000-DFFFFF   R     xxxxxxxx xxxxxxxx   Question ROM bank 2
    E00000-EFFFFF   R     xxxxxxxx xxxxxxxx   Question ROM bank 3
    F00000-FFFFFF   R     xxxxxxxx xxxxxxxx   Question ROM bank 4
    ========================================================================
    Interrupts:
        IRQ5 = MC68681 1
        IRQ6 = Watchdog
        IRQ7 = Power failure detect
    ========================================================================

****************************************************************************/
#include "cpu/m68000/m68000.h"
/* fruit machines only */
#include "video/awpvid.h"
#include "drivers/jpmimpct.c"
#include "machine/steppers.h"
#include "machine/roc10937.h"
#include "machine/8255ppi.h"

/*************************************
 *
 *  Statics
 *
 *************************************/

static int optic_pattern;
static int payen;

/*************************************
 *
 *  Initialisation
 *
 *************************************/

static READ8_DEVICE_HANDLER( hopper_b_r )
{
	//?
	return 0;
}

static READ8_DEVICE_HANDLER( hopper_c_r )
{
	//?
	return 0;
}

static WRITE8_DEVICE_HANDLER( payen_a_w )
{
	if ( data )
	{
		if ( data & 0x10 )
		payen = 1;
		else
		payen = 0;
    }
}

static WRITE8_DEVICE_HANDLER( display_c_w )
{
	//?
}

static const ppi8255_interface ppi8255_intf[1] =
{
	{
		DEVCB_NULL,
		DEVCB_HANDLER(hopper_b_r),
		DEVCB_HANDLER(hopper_c_r),
		DEVCB_HANDLER(payen_a_w),
		DEVCB_NULL,
		DEVCB_HANDLER(display_c_w)
	}
};

static MACHINE_RESET( impctawp )
{
	memset(&duart_1, 0, sizeof(duart_1));

	duart_1_timer = timer_alloc(machine, duart_1_timer_event, NULL);

	/* Reset states */
	duart_1_irq = 0;

	state_save_register_global(machine, duart_1_irq);
	state_save_register_global(machine, touch_cnt);
	state_save_register_global_array(machine, touch_data);

	/* TODO! */
	state_save_register_global(machine, duart_1.ISR);
	state_save_register_global(machine, duart_1.IMR);
	state_save_register_global(machine, duart_1.CT);

	ROC10937_init(0, MSC1937,0);
	ROC10937_reset(0);	/* reset display1 */
	stepper_config(machine, 0, &starpoint_interface_48step);
	stepper_config(machine, 1, &starpoint_interface_48step);
	stepper_config(machine, 2, &starpoint_interface_48step);
	stepper_config(machine, 3, &starpoint_interface_48step);
	stepper_config(machine, 4, &starpoint_interface_48step);
	stepper_config(machine, 5, &starpoint_interface_48step);
	stepper_config(machine, 6, &starpoint_interface_48step);
}
/*************************************
 *
 *  I/O handlers
 *
 *************************************/

/*
 *  0: DIP switches
 *  1: Percentage key
 *  2: Lamps + switches (J10)
 *  3: Lamps + switches (J10)
 *  4: Lamps + switches (J10)
 *      ---- ---x   Back door
 *      ---- --x-   Cash door
 *      ---- -x--   Refill key
 *  5: Lamps + switches (J9)
 *  6: Lamps + switches (J9)
 *  7: Lamps + switches (J9)
 *  8: Payslides
 *  9: Coin mechanism
 */
static READ16_HANDLER( optos_r )
{
	int i;

	for (i=0; i<6; i++)
	{
		if ( stepper_optic_state(i) ) optic_pattern |= (1 << i);
		else                          optic_pattern &= ~(1 << i);
	}
	return optic_pattern;
}

/*************************************
 *
 *  Mysterious stuff
 *
 *************************************/

static WRITE16_HANDLER( jpmioawp_w )
{
	int i;
	UINT64 cycles  = cpu_get_total_cycles(space->cpu);
	switch (offset)
	{
		case 0x00:
		{
			output_set_value("PWRLED",!(data&0x100));
			output_set_value("STATLED",!(data&0x200));
			break;
		}


		case 0x02:
		{
			for (i=0; i<4; i++) 
			{
				stepper_update(i, (data >> i)& 0x0F );
			}
			break;
		}
		case 0x04:
		{
			for (i=0; i<2; i++) 
			{
				stepper_update(i, (data >> (i + 4)& 0x0F ));
			}
			break;
		}
		case 0x06:
		{
			if ( data & 0x10 )
			{   // PAYEN ?
				if ( data & 0xf )
				{
			//      slide = 1;
				}
				else
				{
				//  slide = 0;
				}
			}
			else
//          slide = 0;
			Mechmtr_update(0, cycles, data >> 10);
			if ( data )
			{
				duart_1.IP &= ~0x10;
			}
			else
			{
				duart_1.IP |= 0x10;
			}
			break;
		}

		case 0x08:
		{
			jpm_draw_lamps(data, lamp_strobe);
			break;
		}

		case 0x0b:
		{
			output_set_digit_value(lamp_strobe,data);
			break;
		}
		case 0x0f:
		{
			if (data & 0x10)
			{
				lamp_strobe = (data & 0x0f);
			}
			break;
		}
	}
}

static READ16_HANDLER( ump_r )
{
	return 0xffff;
}

/*************************************
 *
 *  Main CPU memory handlers
 *
 *************************************/
static ADDRESS_MAP_START( awp68k_program_map, ADDRESS_SPACE_PROGRAM, 16 )
	AM_RANGE(0x00000000, 0x000fffff) AM_ROM
	AM_RANGE(0x00100000, 0x001fffff) AM_ROM
	AM_RANGE(0x00400000, 0x00403fff) AM_RAM AM_BASE(&generic_nvram16) AM_SIZE(&generic_nvram_size)
	AM_RANGE(0x00480000, 0x0048001f) AM_READWRITE(duart_1_r, duart_1_w)
	AM_RANGE(0x00480020, 0x00480033) AM_READ(inputs1_r)
	AM_RANGE(0x00480034, 0x00480035) AM_READ(ump_r)
	AM_RANGE(0x00480040, 0x00480041) AM_READ(optos_r)
	AM_RANGE(0x00480060, 0x00480067) AM_DEVREADWRITE8(PPI8255, "ppi8255_0", ppi8255_r, ppi8255_w, 0xff00)
	AM_RANGE(0x004800a0, 0x004800af) AM_READWRITE(jpmio_r, jpmioawp_w)
	AM_RANGE(0x004800e0, 0x004800e1) AM_WRITE(unk_w)
	AM_RANGE(0x004801dc, 0x004801dd) AM_READ(unk_r)
	AM_RANGE(0x004801de, 0x004801df) AM_READ(unk_r)
	AM_RANGE(0x00480080, 0x00480081) AM_WRITE(upd7759_w)
	AM_RANGE(0x00480082, 0x00480083) AM_WRITE(volume_w)
	AM_RANGE(0x00480084, 0x00480085) AM_READ(upd7759_r)
//	AM_RANGE(0x004801e0, 0x004801ff) AM_READWRITE(duart_2_r, duart_2_w)
//	AM_RANGE(0x00800000, 0x00800007) AM_READWRITE(m68k_tms_r, m68k_tms_w)
	AM_RANGE(0x00c00000, 0x00cfffff) AM_ROM
	AM_RANGE(0x00d00000, 0x00dfffff) AM_ROM
	AM_RANGE(0x00e00000, 0x00efffff) AM_ROM
	AM_RANGE(0x00f00000, 0x00ffffff) AM_ROM
ADDRESS_MAP_END

/*************************************
 *
 *  Machine driver
 *
 *************************************/

static MACHINE_DRIVER_START( impctawp )
	MDRV_CPU_ADD("main",M68000, 8000000)
	MDRV_CPU_PROGRAM_MAP(awp68k_program_map, 0)

	MDRV_QUANTUM_TIME(HZ(30000))

	MDRV_MACHINE_RESET(impctawp)
	MDRV_NVRAM_HANDLER(generic_0fill)

	MDRV_PPI8255_ADD( "ppi8255_0", ppi8255_intf[0] )

	MDRV_SPEAKER_STANDARD_MONO("mono")
	MDRV_SOUND_ADD("upd",UPD7759, UPD7759_STANDARD_CLOCK)
	MDRV_SOUND_ROUTE(ALL_OUTPUTS, "mono", 0.50)
	MDRV_DEFAULT_LAYOUT(layout_awpvid16)
MACHINE_DRIVER_END

/*************************************
 *
 *  ROM definition(s)
 *
 *************************************/

ROM_START( thndbrds )
	ROM_REGION( 0x1000000, "main", 0 )
	ROM_LOAD16_BYTE( "thbi-3_1.bin", 0x000000, 0x020000, CRC(ecbdd85c) SHA1(626306a72d658af6ecfa4b65212cf8aa35630539) )
	ROM_LOAD16_BYTE( "thbi-3_2.bin", 0x000001, 0x020000, CRC(61e98f71) SHA1(315044637bde8ab862af08bf9825917c87fcc77d) )

	ROM_FILL(						 0x040000, 0x100000, 0xff )

	ROM_REGION( 0x80000, "upd", 0 )
	ROM_LOAD( "tbcl-snd.bin", 0x000000, 0x80000, CRC(1cc197be) SHA1(2247aa1a0e6aab7389b3222f373890f54e907361) )
ROM_END

/*************************************
 *
 *  Game driver(s)
 *
 *************************************/

GAME( 199?, thndbrds, 0,      impctawp, trivialp, 0, ROT0, "JPM", "Thunderbirds", GAME_NOT_WORKING )
